SystemOrganization addCategory: #'FS-Help'!

CustomHelp subclass: #HelpOnFilesystem
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Help'!

HelpOnFilesystem subclass: #FilesystemHowTo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FS-Help'!

!FilesystemHowTo class methodsFor: 'accessing' stamp: 'MaxLeske 8/14/2011 22:30'!
bookName
	^ 'How to use Filesystem'! !

!FilesystemHowTo class methodsFor: 'pages' stamp: 'MaxLeske 8/14/2011 23:11'!
fileAndDirectoryOperations
	"This method was automatically generated. Edit it using:"
	"FilesystemHowTo edit: #fileAndDirectoryOperations"
	^HelpTopic
		title: 'File and directory operations'
		contents: 
'** How to use Filesystem - File and directory operations **
=====================================================

Files and directories can be copied between references in the same filesystem (first line of next example) or even across different types of filesystems (second line of next example) such as between a disk filesystem and a memory filesystem:
	(working / ''letter.txt'') copyTo: (working / ''letter_backup.txt'').

	memory := FSFilesystem inMemory.
	(working / ''letter.txt'') copyTo: (memory / ''letter_backup.txt'').
	memory children first basename.				" --> ''letter.txt'' "	
					
Supposing one wanted to backup an entire directory there needs to be a suitable target directory. Since there are no directories in the memory filesystem just created we will now create one to hold the mentioned backup:
	backup := memory / ''cache-backup''.
	backup createDirectory.	
		
Finally, we copy the directory to be backed up to the destination directory:
   cache copyAllTo: backup.

The target directory would also have been created automatically by the message #copyAllTo: had it not existed already. From time to time the backup needs to be cleaned up. In this case, we no longer need the backup of the file "letter.txt":
	(memory / ''letter_backup.txt'') delete.

A complete directory tree can be deleted by sending deleteAll:
	backup deleteAll.!!' readStream nextChunkText! !

!FilesystemHowTo class methodsFor: 'pages' stamp: 'MaxLeske 8/25/2011 19:33'!
filesystemsAndReferences
	"This method was automatically generated. Edit it using:"
	"FilesystemHowTo edit: #filesystemsAndReferences"
	^HelpTopic
		title: 'Filesystems and references'
		contents: 
'** How to use Filesystem - Filesystems and references **
=====================================================

The class "FSFilesystem" is the entrypoint for creating and addressing different filesystems. An instance of the filesystem of the current platform can be retrieved by evaluating the following in a workspace:
   disk := FSFilesystem disk.

An FSFilesystem object answers FSReference objects that describe the relationship between a resource (e.g. a file or directory) inside a filesystem and the filesystem itself. We now ask the filesystem for a reference to the working directory and assign it to the variable "working". The working directory is the directory containing the running image.
   working := disk workingDirectory.

Sending the message #root to the filesystem object answers a reference to the root of the filesystem ("/" in Unix, "C:\" in Windows usually). But since we do not want to corrupt any data and permissions might be a problem we will use the working directory instead for further examples.!!' readStream nextChunkText! !

!FilesystemHowTo class methodsFor: 'pages' stamp: 'MaxLeske 8/14/2011 22:59'!
navigatingAFilesystem
	"This method was automatically generated. Edit it using:"
	"FilesystemHowTo edit: #navigatingAFilesystem"
	^HelpTopic
		title: 'Navigating a filesystem'
		contents: 
'** How to use Filesystem - Navigating a filesystem **
=====================================================

The reference assigned to the variable "working" (see tutorial "Filesystems and references") allows us to browse the filesystem. We list the entries of a directory by sending #children to the reference:
	working children.

Sending #allChildren to the reference answers a collection of all entries in all subdirectories recursively:
   working allChildren.

The slash operator answers a reference to a specific file or directory within the working directory:
	cache := working / ''package-cache''.
	
Navigating back to the parent is easy. We send #parent to the reference. We expect
the parent to be the working directory and test for that:
	parentOfCache := cache parent.
	parentOfCache = working.					" --> true "
						
The slash operator interprets the string sent to it as the name of the directory to navigate to. If the string argument contains a slash it is interpreted as part of the directory name and not as a separator. To remedy the situation references understand a second navigation message called #resolve:. Resolve expects a string argument in the form used by the "cd" command known from the Unix command line (the same syntax applies to Windows systems):
	disk working resolve: ''/''.					" the same as ''disk root'' "
	disk working resolve: ''.''.						" the same as ''disk working'' "
	disk working resolve: ''/home/leske''.		" an absolute path to a directory or file "
	disk working resolve: ''../projects''.			" a relative path from the working directory "
		
Evaluating the following expressions reveals information about the cache directory:
	cache exists.								" --> true "
	cache isFile.								" --> false "
	cache isDirectory.							" --> true "
	cache basename.							" --> ''package-cache'' "
								
Even more information is available via the filesystem entry. The entry is an instance of FSDirectoryEntry. Every reference has an associated entry holding the header information:
	cache entry creation.						" --> 2010-09-14T10:34:31+00:00 "
	cache entry modification.					" --> 2010-09-14T10:34:31+00:00 "
	cache entry size.							" --> 0 (directories have size 0) "
								
The framework also supports locations (note that locations may not work on your system. They''re still work in progress); they are late-bound references that point to a file or directory. When asking to perform a concrete operation a location behaves the same way as a reference. Filesystem supports the following locations:
	FSLocator desktop.
	FSLocator home.
	FSLocator image.
	FSLocator changes.
	FSLocator imageDirectory.
	FSLocator vmBinary.
	FSLocator vmDirectory.	

A location will dynamically adapt and always point to the place expected even when moving the image to another platform.!!' readStream nextChunkText! !

!FilesystemHowTo class methodsFor: 'pages' stamp: 'MaxLeske 8/14/2011 23:14'!
openingReadAndWriteStreams
	"This method was automatically generated. Edit it using:"
	"FilesystemHowTo edit: #openingReadAndWriteSTreams"
	^HelpTopic
		title: 'Opening read and write streams'
		contents: 
'** How to use Filesystem - Opening read and write streams **
=====================================================

FSReference provides easy access to streams:
	working := FSLocator imageDirectory.
	stream := (working / ''letter.txt'') writeStream.
	stream nextPutAll: ''Hello Alice''.
	stream close.
	stream := (working / ''letter.txt'') readStream.
	stream contents.									" --> ''Hello Alice'' "
	stream close.
	
Note that #writeStream overrides any existing file and that #readStream throws an exception if the file does not exist. There are also short forms available which ensure the correct closing of the stream:
	working / ''letter.txt'' writeStreamDo: [ :stream | stream nextPutAll: ''Hello Alice'' ].
	working / ''letter.txt'' readStreamDo: [ :stream | stream contents ].!!' readStream nextChunkText! !

!FilesystemHowTo class methodsFor: 'pages' stamp: 'MaxLeske 8/14/2011 23:13'!
otherFilesystems
	"This method was automatically generated. Edit it using:"
	"FilesystemHowTo edit: #otherFilesystems"
	^HelpTopic
		title: 'Other filesystems'
		contents: 
'** How to use Filesystem - Other filesystem **
=====================================================

The filesystem framework is easily extensible and there are several implementations including a ZIP, a cURL and a Git filesystem. All of these implementations comply with the public protocol of the framework and therefore it does not matter to the user on which filesystem he or she is working. Some filesystems specify additional messages to provide access to functionality beyond the scope of a simple filesystem. The ZIP filesystem for example defines the message #close which needs to be sent to the instance of a ZIP archive to flush it.!!' readStream nextChunkText! !

!FilesystemHowTo class methodsFor: 'accessing' stamp: 'MaxLeske 8/14/2011 23:15'!
pages
	^#(
		filesystemsAndReferences 
		navigatingAFilesystem 
		openingReadAndWriteStreams 
		fileAndDirectoryOperations 
		otherFilesystems)! !

!HelpOnFilesystem class methodsFor: 'accessing' stamp: 'MaxLeske 8/14/2011 22:20'!
bookName
	^ 'Filesystem'! !

!HelpOnFilesystem class methodsFor: 'accessing' stamp: 'MaxLeske 8/14/2011 22:28'!
pages
	^ #()! !
